@@ -7,8 +7,7 @@ setClass("GroupPars",
                         parnum='numeric',
                         nfact='numeric',
                         gradient='numeric',
-                        hessian='matrix',
-                        method='character',
+                        hessian='matrix',                        
                         lbound='numeric',
                         ubound='numeric',
                         n.prior.mu='numeric',
@@ -25,11 +24,10 @@ setClass("AllItemsClass",
                         parnum='numeric',
                         nfact='numeric',
                         dat='matrix',
+                        ncat='numeric',
                         rs='matrix',
-                        bfactor='logical',
                         gradient='numeric',
-                        hessian='matrix',
-                        method='character',
+                        hessian='matrix',                        
                         lbound='numeric',
                         ubound='numeric',
                         n.prior.mu='numeric',
@@ -39,23 +37,19 @@ setClass("AllItemsClass",
                         'VIRTUAL')
 )
 
-setClass("NullModel", contains = 'AllItemsClass')
-
 setClass("dich", contains = 'AllItemsClass')
 
-setClass("graded", contains = 'AllItemsClass',
-         representation(ncat='numeric'))
+setClass("graded", contains = 'AllItemsClass')
 
-setClass("gpcm", contains = 'AllItemsClass',
-         representation(ncat='numeric'))
+setClass("rating", contains = 'AllItemsClass')
 
-setClass("nominal", contains = 'AllItemsClass',
-         representation(ncat='numeric'))
+setClass("gpcm", contains = 'AllItemsClass')
+
+setClass("nominal", contains = 'AllItemsClass')
 
 setClass("partcomp", contains = 'AllItemsClass')
 
-setClass("mcm", contains = 'AllItemsClass',
-         representation(ncat='numeric'))
+setClass("mcm", contains = 'AllItemsClass')
 
 #--------------------------------------------------------------------------
 
