@@ -15,6 +15,14 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "print",
+    signature = signature(x = 'rating'),
+    definition = function(x, ...){
+        cat('Item object of class:', class(x))
+    }
+)
+
 setMethod(
     f = "print",
     signature = signature(x = 'gpcm'),
@@ -63,6 +71,14 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "show",
+    signature = signature(object = 'rating'),
+    definition = function(object){
+        print(object)
+    }
+)
+
 setMethod(
     f = "show",
     signature = signature(object = 'gpcm'),
@@ -104,8 +120,7 @@ setMethod(
         u <- x@par[length(x@par)]
         g <- x@par[length(x@par)-1]
         d <- x@par[length(x@par)-2]
-        a <- x@par[1:x@nfact]
-        if(x@bfactor) a <- a[x@est[1:x@nfact]]
+        a <- x@par[1:x@nfact]        
         P <- P.mirt(a=a, d=d, Theta=Theta, g=g, u=u)
         return(cbind(1.0 - P, P))
     }
@@ -115,20 +130,32 @@ setMethod(
     f = "ProbTrace",
     signature = signature(x = 'graded', Theta = 'matrix'),
     definition = function(x, Theta, itemexp = TRUE){                  
-        a <- x@par[1:x@nfact]
-        if(x@bfactor) a <- a[x@est[1:x@nfact]]
+        a <- x@par[1:x@nfact]        
         d <- x@par[(x@nfact+1):length(x@par)]
         P <- P.poly(a=a, d=d, Theta=Theta, itemexp=itemexp)
         return(P)
     }
 )
 
+
+setMethod(
+    f = "ProbTrace",
+    signature = signature(x = 'rating', Theta = 'matrix'),
+    definition = function(x, Theta, itemexp = TRUE){
+        nfact <- x@nfact
+        a <- x@par[1:nfact]
+        d <- x@par[(nfact+1):(length(x@par)-1)]
+        t <- x@par[length(x@par)]
+        P <- P.poly(a=a, d=(d + t), Theta=Theta, itemexp=itemexp)
+        return(P)
+    }
+)
+
 setMethod(
     f = "ProbTrace",
     signature = signature(x = 'gpcm', Theta = 'matrix'),
     definition = function(x, Theta){                  
-        a <- x@par[1:x@nfact]
-        if(x@bfactor) a <- a[x@est[1:x@nfact]]
+        a <- x@par[1:x@nfact]        
         d <- x@par[-(1:x@nfact)]
         P <- P.gpcm(a=a, d=d, Theta=Theta)
         return(P)
@@ -139,8 +166,7 @@ setMethod(
     f = "ProbTrace",
     signature = signature(x = 'nominal', Theta = 'matrix'),
     definition = function(x, Theta){         
-        a <- x@par[1:x@nfact]
-        if(x@bfactor) a <- a[x@est[1:x@nfact]]
+        a <- x@par[1:x@nfact]        
         ak <- x@par[(x@nfact+1):(x@nfact + x@ncat)]
         d <- x@par[length(x@par):(length(x@par) - x@ncat + 1)]
         P <- P.nominal(a=a, ak=ak, d=d, Theta=Theta)
@@ -156,9 +182,7 @@ setMethod(
         a <- x@par[1:nfact]
         d <- x@par[(nfact+1):(length(x@par)-2)]
         g <- x@par[length(x@par)-1]
-        u <- x@par[length(x@par)]
-        if(x@bfactor) a <- a[x@est[1:nfact]]
-        if(x@bfactor) d <- d[x@est[(nfact+1):(nfact*2)]]        
+        u <- x@par[length(x@par)]        
         P <- P.comp(a=a, d=d, Theta=Theta, g=g, u=u)
         return(cbind(1.0 - P, P))
     }
@@ -168,8 +192,7 @@ setMethod(
     f = "ProbTrace",
     signature = signature(x = 'mcm', Theta = 'matrix'),
     definition = function(x, Theta){    
-        a <- x@par[1:x@nfact]
-        if(x@bfactor) a <- a[x@est[1:x@nfact]]
+        a <- x@par[1:x@nfact]        
         ind <- x@nfact + 1
         ak <- x@par[ind:(ind + x@ncat - 1)]
         ind <- ind + x@ncat
@@ -186,10 +209,12 @@ setMethod(
 setMethod(
     f = "LogLik",
     signature = signature(x = 'dich', Theta = 'matrix'),
-    definition = function(x, Theta){          
+    definition = function(x, Theta, EM=FALSE, prior=NULL){          
         itemtrace <- ProbTrace(x=x, Theta=Theta)        
         itemtrace[itemtrace < 1e-8] <- 1e-8
-        LL <- (-1) * sum(x@rs * log(itemtrace))
+        Prior <- rep(1, nrow(itemtrace))
+        if(EM) Prior <- prior
+        LL <- (-1) * sum(x@rs * log(itemtrace) * Prior)
         if(any(!is.nan(x@n.prior.mu))){
             ind <- !is.nan(x@n.prior.mu)
             val <- x@par[ind]
@@ -215,10 +240,43 @@ setMethod(
 setMethod(
     f = "LogLik",
     signature = signature(x = 'graded', Theta = 'matrix'),
-    definition = function(x, Theta){          
+    definition = function(x, Theta, EM = FALSE, prior = NULL){          
         itemtrace <- ProbTrace(x=x, Theta=Theta)
         itemtrace[itemtrace < 1e-8] <- 1e-8
-        LL <- (-1) * sum(x@rs * log(itemtrace))
+        Prior <- rep(1, nrow(itemtrace))
+        if(EM) Prior <- prior
+        LL <- (-1) * sum(x@rs * log(itemtrace) * Prior)
+        if(any(!is.nan(x@n.prior.mu))){
+            ind <- !is.nan(x@n.prior.mu)
+            val <- x@par[ind]
+            u <- x@n.prior.mu[ind]
+            s <- x@n.prior.sd[ind]
+            for(i in 1:length(val))            
+                LL <- LL - log(dnorm(val[i], u[i], s[i]))
+        }
+        if(any(!is.nan(x@b.prior.alpha))){
+            ind <- !is.nan(x@b.prior.alpha)
+            val <- x@par[ind]
+            a <- x@b.prior.alpha[ind]
+            b <- x@b.prior.beta[ind]
+            for(i in 1:length(val)){            
+                tmp <- dbeta(val[i], a[i], b[i])
+                LL <- LL - log(ifelse(tmp == 0, 1, tmp))
+            }
+        }
+        return(LL)
+    }
+)
+
+setMethod(
+    f = "LogLik",
+    signature = signature(x = 'rating', Theta = 'matrix'),
+    definition = function(x, Theta, EM = FALSE, prior = NULL){          
+        itemtrace <- ProbTrace(x=x, Theta=Theta)
+        itemtrace[itemtrace < 1e-8] <- 1e-8
+        Prior <- rep(1, nrow(itemtrace))
+        if(EM) Prior <- prior
+        LL <- (-1) * sum(x@rs * log(itemtrace) * Prior)
         if(any(!is.nan(x@n.prior.mu))){
             ind <- !is.nan(x@n.prior.mu)
             val <- x@par[ind]
@@ -244,10 +302,12 @@ setMethod(
 setMethod(
     f = "LogLik",
     signature = signature(x = 'gpcm', Theta = 'matrix'),
-    definition = function(x, Theta){          
+    definition = function(x, Theta, EM = FALSE, prior = NULL){          
         itemtrace <- ProbTrace(x=x, Theta=Theta)
         itemtrace[itemtrace < 1e-8] <- 1e-8
-        LL <- (-1) * sum(x@rs * log(itemtrace))
+        Prior <- rep(1, nrow(itemtrace))
+        if(EM) Prior <- prior
+        LL <- (-1) * sum(x@rs * log(itemtrace) * Prior)
         if(any(!is.nan(x@n.prior.mu))){
             ind <- !is.nan(x@n.prior.mu)
             val <- x@par[ind]
@@ -273,10 +333,12 @@ setMethod(
 setMethod(
     f = "LogLik",
     signature = signature(x = 'nominal', Theta = 'matrix'),
-    definition = function(x, Theta){          
+    definition = function(x, Theta, EM = FALSE, prior = NULL){          
         itemtrace <- ProbTrace(x=x, Theta=Theta)
         itemtrace[itemtrace < 1e-8] <- 1e-8
-        LL <- (-1) * sum(x@rs * log(itemtrace))
+        Prior <- rep(1, nrow(itemtrace))
+        if(EM) Prior <- prior
+        LL <- (-1) * sum(x@rs * log(itemtrace) * Prior)
         if(any(!is.nan(x@n.prior.mu))){
             ind <- !is.nan(x@n.prior.mu)
             val <- x@par[ind]
@@ -302,10 +364,12 @@ setMethod(
 setMethod(
     f = "LogLik",
     signature = signature(x = 'partcomp', Theta = 'matrix'),
-    definition = function(x, Theta){          
+    definition = function(x, Theta, EM = FALSE, prior = NULL){          
         itemtrace <- ProbTrace(x=x, Theta=Theta)
         itemtrace[itemtrace < 1e-8] <- 1e-8
-        LL <- (-1) * sum(x@rs * log(itemtrace))
+        Prior <- rep(1, nrow(itemtrace))
+        if(EM) Prior <- prior
+        LL <- (-1) * sum(x@rs * log(itemtrace) * Prior)
         if(any(!is.nan(x@n.prior.mu))){
             ind <- !is.nan(x@n.prior.mu)
             val <- x@par[ind]
@@ -331,10 +395,12 @@ setMethod(
 setMethod(
     f = "LogLik",
     signature = signature(x = 'mcm', Theta = 'matrix'),
-    definition = function(x, Theta){          
+    definition = function(x, Theta, EM = FALSE, prior = NULL){          
         itemtrace <- ProbTrace(x=x, Theta=Theta)
         itemtrace[itemtrace < 1e-8] <- 1e-8
-        LL <- (-1) * sum(x@rs * log(itemtrace))
+        Prior <- rep(1, nrow(itemtrace))
+        if(EM) Prior <- prior
+        LL <- (-1) * sum(x@rs * log(itemtrace) * Prior)
         if(any(!is.nan(x@n.prior.mu))){
             ind <- !is.nan(x@n.prior.mu)
             val <- x@par[ind]
@@ -360,7 +426,7 @@ setMethod(
 setMethod(
     f = "LogLik",
     signature = signature(x = 'GroupPars', Theta = 'matrix'),
-    definition = function(x, Theta, pars, tabdata, itemloc){
+    definition = function(x, Theta, pars, tabdata, itemloc, EM = TRUE){
         r <- tabdata[, ncol(tabdata)]
         gpars <- ExtractGroupPars(x)
         mu <- gpars$gmeans
@@ -395,6 +461,16 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "ExtractLambdas",
+    signature = signature(x = 'rating'),
+    definition = function(x){          
+        par <- x@par
+        a <- par[1:x@nfact]
+        a        
+    }
+)
+
 setMethod(
     f = "ExtractLambdas",
     signature = signature(x = 'gpcm'),
@@ -456,6 +532,16 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "ExtractZetas",
+    signature = signature(x = 'rating'),
+    definition = function(x){          
+        par <- x@par
+        d <- par[-c(1:x@nfact, length(par))]
+        d
+    }
+)
+
 setMethod(
     f = "ExtractZetas",
     signature = signature(x = 'gpcm'),
@@ -470,7 +556,6 @@ setMethod(
     f = "ExtractZetas",
     signature = signature(x = 'nominal'),
     definition = function(x){          
-        par <- x@par
         d <- x@par[length(x@par):(length(x@par) - x@ncat + 1)]
         d        
     }
@@ -480,8 +565,7 @@ setMethod(
     f = "ExtractZetas",
     signature = signature(x = 'partcomp'),
     definition = function(x){          
-        par <- x@par
-        d <- x@par[(nfact+1):(length(x@par)-2)]
+        d <- x@par[(x@nfact+1):(length(x@par)-2)]
         d
     }
 )
@@ -490,7 +574,6 @@ setMethod(
     f = "ExtractZetas",
     signature = signature(x = 'mcm'),
     definition = function(x){          
-        par <- x@par
         d <- x@par[(x@nfact + x@ncat +1):(x@nfact - x@ncat*2)]
         d        
     }
@@ -528,6 +611,23 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "ItemInfo",
+    signature = signature(x = 'rating', Theta = 'matrix', cosangle = 'numeric'),
+    definition = function(x, Theta, cosangle = 1){
+        P <- ProbTrace(x, Theta, itemexp = FALSE) 
+        a <- ExtractLambdas(x)
+        A <- sum((a * cosangle)^2)
+        info <- 0
+        for(i in 1:(ncol(P)-1)){
+            w1 <- P[,i]*(1-P[,i]) * A
+            w2 <- P[,i+1]*(1-P[,i+1]) * A
+            info <- info + ((w1 - w2)^2) / (P[,i] - P[,i+1])             
+        }    
+        info      
+    }
+)
+
 setMethod(
     f = "ItemInfo",
     signature = signature(x = 'gpcm', Theta = 'matrix', cosangle = 'numeric'),
@@ -572,11 +672,13 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'dich', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE){        
+    definition = function(x, Theta, EM = FALSE, prior = NULL){        
         f <- 1
         dat <- x@dat[ ,2]
+        Prior <- rep(1, length(dat))
         if(EM){
-            dat <- x@rs[,2]
+            Prior <- prior
+            dat <- x@rs[,2] 
             f <- rowSums(x@rs)
         }
         nfact <- x@nfact
@@ -586,42 +688,54 @@ setMethod(
         d <- x@par[parlength - 2]
         a <- x@par[1:nfact]        
         P <- P.mirt(a, d, Theta, g, u)
+        P[P < 1e-8] <- 1e-8
+        P[P > .99999999] <- .99999999
         Q <- 1 - P        
         hess <- matrix(0,nfact+3, nfact+3)						
         if(x@par[parlength] == 1){ #'3PL'            	
             Pstar <- P.mirt(a,d,Theta,0,1)		
+            Pstar[Pstar < 1e-8] <- 1e-8
+            Pstar[Pstar > .99999999] <- .99999999
             Qstar <- 1 - Pstar
             da <- rep(0,nfact)	
-            dd <- sum((1-g)*Pstar*Qstar*(dat/P - (f-dat)/Q))
-            dc <- sum(Qstar*(dat/P - (f-dat)/Q))
+            dd <- sum((1-g)*Pstar*Qstar*(dat/P - (f-dat)/Q)*Prior)
+            dc <- sum(Qstar*(dat/P - (f-dat)/Q)*Prior)
             for(i in 1:nfact){
-                da[i] <- sum(Theta[,i]*Pstar*Qstar*(1-g)*(dat/P - (f-dat)/Q))
+                da[i] <- sum(Theta[,i]*Pstar*Qstar*(1-g)*(dat/P - (f-dat)/Q)*Prior)
             }
             grad <- c(da,dd,dc,0)				
             gloc <- nfact+2
             const1 <- (dat/P - (f-dat)/Q)*(Qstar-Pstar)
             const2 <- (dat/P^2 + (f-dat)/Q^2)	
             hess[nfact+1,nfact+1] <- sum((1-g)*Pstar*Qstar*(const1 - 
-                Pstar*Qstar*(1-g)*const2))		
-            hess[gloc,gloc] <- -sum(Qstar^2 *(dat/P^2 + (f-dat)/Q^2))
-            hess[gloc,nfact+1] <- hess[nfact+1,gloc] <- sum(-Pstar*Qstar*((dat/P - (f-dat)/Q) + Qstar*(1-g)*const2)) 
+                Pstar*Qstar*(1-g)*const2)*Prior)		
+            hess[gloc,gloc] <- -sum(Qstar^2 *(dat/P^2 + (f-dat)/Q^2)*Prior)
+            hess[gloc,nfact+1] <- hess[nfact+1,gloc] <- sum(-Pstar*Qstar*((dat/P - (f-dat)/Q) + 
+                Qstar*(1-g)*const2)*Prior) 
             for(i in 1:nfact){
                 hess[nfact+1,i] <- hess[i,nfact+1] <- sum((1-g)*Theta[,i]*Pstar*Qstar*(const1 - 
-                    Pstar*Qstar*(1-g)*const2))			
+                    Pstar*Qstar*(1-g)*const2)*Prior)			
                 hess[gloc,i] <- hess[i,gloc] <- sum(-Theta[,i]*Pstar*Qstar*((dat/P - (f-dat)/Q) + 
-                    Qstar*(1-g)*const2))		
+                    Qstar*(1-g)*const2)*Prior)		
                 for(j in 1:nfact){
                     if(i == j)
                         hess[i,i] <- sum(Theta[,i]^2 *Pstar*Qstar*(1-g)*(const1 - 
-                            (1-g)*Pstar*Qstar*const2))
+                            (1-g)*Pstar*Qstar*const2)*Prior)
                     if(i < j)
                         hess[i,j] <- hess[j,i] <- sum(Theta[,i]*Theta[,j] *Pstar*Qstar*(1-g)*
-                            (const1 - (1-g)*Pstar*Qstar*const2))					
+                            (const1 - (1-g)*Pstar*Qstar*const2)*Prior)					
                 }
             }	
-        } else { #4PL            
-            grad <- numDeriv::grad(L, x@par, obj=x, Theta=Theta)
-            hess <- numDeriv::hessian(L, x@par, obj=x, Theta=Theta)            
+        } else { #4PL 
+            grad <- rep(0, length(x@par))
+            hess <- matrix(0, length(x@par), length(x@par))
+            if(EM){                
+                grad[x@est] <- numDeriv::grad(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)
+                hess[x@est, x@est] <- numDeriv::hessian(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)       
+                return(list(grad = grad, hess = hess))            
+            }            
+            grad[x@est] <- numDeriv::grad(L, x@par[x@est], obj=x, Theta=Theta)
+            hess[x@est, x@est] <- numDeriv::hessian(L, x@par[x@est], obj=x, Theta=Theta)            
             return(list(grad=grad, hess=hess))
         }       
         if(any(!is.nan(x@n.prior.mu))){
@@ -659,15 +773,21 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'graded', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE){ 
-        dat <- x@dat
-        if(EM) dat <- x@rs                    
+    definition = function(x, Theta, EM = FALSE, prior = NULL){         
+        grad <- rep(0, length(x@par))
+        hess <- matrix(0, length(x@par), length(x@par))        
+        dat <- x@dat 
+        Prior <- rep(1, nrow(dat))
+        if(EM){
+            dat <- x@rs / sum(x@rs)                          
+            Prior <- prior
+        } 
         nfact <- x@nfact
         a <- x@par[1:nfact]
         d <- x@par[-(1:nfact)]
         nd <- length(d)    			
         P <- P.poly(a, d,Theta)			    	
-        ret <- .Call("dparsPoly", P, Theta, dat, nd) #switch the order
+        ret <- .Call("dparsPoly", P, Theta, Prior, dat, nd) 
         grad <- c(ret$grad[-(1:nd)], ret$grad[1:nd])
         hess <- matrix(0,nfact+nd,nfact+nd)
         hess[1:nfact,1:nfact] <- ret$hess[-(1:nd),-(1:nd)]
@@ -706,10 +826,27 @@ setMethod(
     }
 )
 
+setMethod(
+    f = "Deriv",
+    signature = signature(x = 'rating', Theta = 'matrix'),
+    definition = function(x, Theta, EM = FALSE, prior = NULL){ 
+        grad <- rep(0, length(x@par))
+        hess <- matrix(0, length(x@par), length(x@par))
+        if(EM){                
+            grad[x@est] <- numDeriv::grad(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)
+            hess[x@est, x@est] <- numDeriv::hessian(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)       
+            return(list(grad = grad, hess = hess))            
+        }        
+        grad[x@est] <- numDeriv::grad(L, x@par[x@est], obj=x, Theta=Theta)
+        hess[x@est, x@est] <- numDeriv::hessian(L, x@par[x@est], obj=x, Theta=Theta)        
+        return(list(grad = grad, hess = hess))
+    }
+)
+
 setMethod(
     f = "Deriv",
     signature = signature(x = 'partcomp', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE){
+    definition = function(x, Theta, EM = FALSE, prior = NULL){
         f <- 1
         r <- x@dat
         if(EM){       
@@ -725,6 +862,10 @@ setMethod(
             grad <- function(a, d, g, u, r, f, Theta){                			
                 P <- P.comp(a,d,Theta,g,1)		
                 Pstar <- P.comp(a,d,Theta,0)		
+                P[P < 1e-8] <- 1e-8
+                P[P > .99999999] <- .99999999
+                Pstar[Pstar < 1e-8] <- 1e-8
+                Pstar[Pstar > .99999999] <- .99999999
                 Qstar <- 1 - Pstar
                 Q <- 1 - P
                 const1 <- (r/P - (f-r)/Q)
@@ -742,7 +883,9 @@ setMethod(
                 nfact <- length(a)
                 hess <- matrix(0, nfact*2 + 2, nfact*2 + 2)                		
                 P <- P.comp(a,d,Theta,g, 1)		
-                Pstar <- P.comp(a,d,Theta,0, 1)		
+                Pstar <- P.comp(a,d,Theta,0, 1)
+                P[P < 1e-8] <- 1e-8
+                Pstart[Pstart < 1e-8] <- 1e-8
                 Qstar <- 1 - Pstar
                 Q <- 1 - P	
                 const1 <- (r/P - (f-r)/Q)
@@ -842,14 +985,14 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'gpcm', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE){
-        if(EM){                
-            grad <- numDeriv::grad(EML, x@par, obj=x, Theta=Theta)
-            hess <- numDeriv::hessian(EML, x@par, obj=x, Theta=Theta)       
-            return(list(grad = grad, hess = hess))            
-        }
+    definition = function(x, Theta, EM = FALSE, prior = NULL){
         grad <- rep(0, length(x@par))
         hess <- matrix(0, length(x@par), length(x@par))
+        if(EM){            
+            grad[x@est] <- numDeriv::grad(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)
+            hess[x@est, x@est] <- numDeriv::hessian(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)       
+            return(list(grad = grad, hess = hess))            
+        }        
         grad[x@est] <- numDeriv::grad(L, x@par[x@est], obj=x, Theta=Theta)
         hess[x@est, x@est] <- numDeriv::hessian(L, x@par[x@est], obj=x, Theta=Theta)        
         return(list(grad = grad, hess = hess))
@@ -859,14 +1002,14 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'nominal', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE){
-        if(EM){                
-            grad <- numDeriv::grad(EML, x@par, obj=x, Theta=Theta)
-            hess <- numDeriv::hessian(EML, x@par, obj=x, Theta=Theta)       
-            return(list(grad = grad, hess = hess))            
-        }
+    definition = function(x, Theta, EM = FALSE, prior = NULL){
         grad <- rep(0, length(x@par))
         hess <- matrix(0, length(x@par), length(x@par))
+        if(EM){                
+            grad[x@est] <- numDeriv::grad(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)
+            hess[x@est, x@est] <- numDeriv::hessian(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)       
+            return(list(grad = grad, hess = hess))            
+        }        
         grad[x@est] <- numDeriv::grad(L, x@par[x@est], obj=x, Theta=Theta)
         hess[x@est, x@est] <- numDeriv::hessian(L, x@par[x@est], obj=x, Theta=Theta)        
         return(list(grad = grad, hess = hess))
@@ -876,14 +1019,14 @@ setMethod(
 setMethod(
     f = "Deriv",
     signature = signature(x = 'mcm', Theta = 'matrix'),
-    definition = function(x, Theta, EM = FALSE){
-        if(EM){                
-            grad <- numDeriv::grad(EML, x@par, obj=x, Theta=Theta)
-            hess <- numDeriv::hessian(EML, x@par, obj=x, Theta=Theta)     
-            return(list(grad = grad, hess = hess))            
-        }
+    definition = function(x, Theta, EM = FALSE, prior = NULL){
         grad <- rep(0, length(x@par))
         hess <- matrix(0, length(x@par), length(x@par))
+        if(EM){                
+            grad[x@est] <- numDeriv::grad(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)
+            hess[x@est, x@est] <- numDeriv::hessian(EML, x@par[x@est], obj=x, Theta=Theta, prior=prior)     
+            return(list(grad = grad, hess = hess))            
+        }        
         grad[x@est] <- numDeriv::grad(L, x@par[x@est], obj=x, Theta=Theta)
         hess[x@est, x@est] <- numDeriv::hessian(L, x@par[x@est], obj=x, Theta=Theta)
         return(list(grad = grad, hess = hess))
@@ -1006,24 +1149,6 @@ P.comp <- function(a, d, Theta, g = 0, u = 1)
     P	
 }
 
-# Trace lines for bfactor
-P.bfactor <- function(a, d, Theta, g, u, patload)
-{ 
-    a <- a[patload]	
-    if(length(d) > 1){
-        ncat <- length(d) + 1
-        nfact <- length(a)
-        Pk <- matrix(0,nrow(Theta),ncat+1)
-        Pk[,1] <- 1	
-        for(i in 1:(ncat-1))			
-            Pk[ ,i+1] <- P.mirt(a, d[i], Theta, 0)				
-        P <- matrix(0,nrow(Theta),ncat)		
-        for(i in ncat:1)
-            P[ ,i] <- Pk[ ,i] - Pk[ ,i+1]		
-    } else P <- .Call("traceLinePts", a, d, g, u, Theta)		
-    return(P)
-}
-
 #d[1] == 0, ak[1] == 0, ak[length(ak)] == length(ak) - 1 
 P.nominal <- function(a, ak, d, Theta){
     ncat <- length(d)
@@ -1054,8 +1179,7 @@ P.mcm <- function(a, ak, d, t, Theta){
     ncat <- length(d)
     nfact <- ncol(Theta)    
     a <- matrix(a)    
-    P <- numerator <- matrix(0, nrow(Theta), ncat)  
-    
+    P <- numerator <- matrix(0, nrow(Theta), ncat)      
     for(i in 1:ncat)
         numerator[ ,i] <- exp(1.702 * ak[i] * (Theta %*% a) + 1.702 * d[i])
     denominator <- rowSums(numerator)
@@ -1117,8 +1241,8 @@ L <- function(par, obj, Theta){
     return(LL)
 }
 
-EML <- function(par, obj, Theta, ...){
+EML <- function(par, obj, Theta, ...){    
     obj@par[obj@est] <- par
-    L <- (-1)*LogLik(x=obj, Theta=Theta, ...)
+    L <- (-1)*LogLik(x=obj, Theta=Theta, EM=TRUE, ...)
     return(L)
 }
