@@ -3,11 +3,9 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                      estLambdas, BFACTOR = FALSE, debug)
     {       
     if(debug == 'LoadPars') browser() 
-    if(any(itemtype[1] == c('Rasch', '1PL') && nfact > 1)) 
-        stop('Rasch and 1PL models can only be estimated for unidimensional models')
-    pars <- list()       
-    RETURNSTARTVALUES <- ifelse(!is.null(startvalues) && startvalues == 'index', TRUE, FALSE)
-    RETURNFREEPARS <- ifelse(!is.null(freepars) && freepars == 'index', TRUE, FALSE)    
+    if(any(itemtype[1] == c('Rasch', '1PL', 'rating') && nfact > 1)) 
+        stop('Rasch, 1PL, and rating scale models can only be estimated for unidimensional models')
+    pars <- list()           
     constr <- c()
     if(!is.null(constrain) && is.list(constrain)) 
         for(i in 1:length(constrain))
@@ -16,17 +14,19 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
     #startvalues
     if(is.null(startvalues) || startvalues =='index'){        
         startvalues <- list()
-        for(i in 1:J){
-            if(itemtype[i] == 'NullModel' && K[i] == 2) val <- c(0,zetas[[i]],0,1)                                
-            if(itemtype[i] == 'NullModel' && K[i] > 2) val <- c(0,zetas[[i]]) 
+        for(i in 1:J){            
             if(any(itemtype[i] == c('Rasch', '1PL')) && K[i] == 2){
-                val <- c(lambdas[i,], zetas[[i]], guess[i], upper[i])
+                val <- c(1/1.702, zetas[[i]], guess[i], upper[i])
                 names(val) <- c(paste('a', 1:nfact, sep=''), 'd', 'g','u')
             }
-            if(any(itemtype[i] == c('Rasch', '1PL')) && K[i] > 2){
-                val <- c(lambdas[i,], zetas[[i]])
+            if(itemtype[i] == 'Rasch' && K[i] > 2){
+                val <- c(1/1.702, 0, zetas[[i]])
                 names(val) <- c(paste('a', 1:nfact, sep=''), paste('d', 0:(K[i]-1), sep=''))
             }
+            if(itemtype[i] == '1PL' && K[i] > 2){
+                val <- c(1/1.702, zetas[[i]])
+                names(val) <- c(paste('a', 1:nfact, sep=''), paste('d', 1:(K[i]-1), sep=''))
+            }
             if(any(itemtype[i] == c('2PL', '3PL', '3PLu', '4PL'))){
                 val <- c(lambdas[i,], zetas[[i]], guess[i], upper[i])
                 names(val) <- c(paste('a', 1:nfact, sep=''), 'd', 'g','u')
@@ -34,6 +34,10 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
             if(itemtype[i] == 'graded'){
                 val <- c(lambdas[i,], zetas[[i]])
                 names(val) <- c(paste('a', 1:nfact, sep=''), paste('d', 1:(K[i]-1), sep=''))    
+            }            
+            if(itemtype[i] == 'grsm'){
+                val <- c(lambdas[i,], zetas[[1]], 0) #first item intercepts
+                names(val) <- c(paste('a', 1:nfact, sep=''), paste('d', 1:(K[i]-1), sep=''), 'c')
             }
             if(itemtype[i] == 'gpcm'){
                 val <- c(lambdas[i,], 0, zetas[[i]])
@@ -53,18 +57,14 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                          rep(1/K[i], K[i]))
                 names(val) <- c(paste('a', 1:nfact, sep=''), paste('ak', 0:(K[i]-1), sep=''), 
                                 paste('d', 0:(K[i]-1), sep=''), paste('t', 0:(K[i]-1), sep=''))                
-            }
+            }            
             startvalues[[i]] <- val
         } 
     }  
     #freepars
     if(is.null(freepars) || freepars == 'index'){
         freepars <- list()
-        for(i in 1:J){
-            if(itemtype[i] == 'NullModel' && K[i] == 2)
-                freepars[[i]] <- c(FALSE,TRUE,FALSE,FALSE)
-            if(itemtype[i] == 'NullModel' && K[i] > 2)    
-                freepars[[i]] <- c(FALSE,rep(TRUE,K[i]-1))            
+        for(i in 1:J){                        
             if(itemtype[i] == 'Rasch' && K[i] == 2)
                 freepars[[i]] <- c(FALSE,TRUE,FALSE,FALSE)            
             if(any(itemtype[i] == c('1PL', '2PL', '3PL', '3PLu', '4PL'))){
@@ -74,8 +74,10 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                 freepars[[i]] <- estpars
             }
             if(itemtype[i] == 'Rasch' && K[i] > 2)            
-                freepars[[i]] <- c(FALSE, rep(TRUE, K[i]))
+                freepars[[i]] <- c(FALSE, FALSE, rep(TRUE, K[i]-1))
             if(itemtype[i] == '1PL' && K[i] > 2)            
+                freepars[[i]] <- c(estLambdas[i, ], rep(TRUE, K[i]))            
+            if(itemtype[i] == 'grsm')
                 freepars[[i]] <- c(estLambdas[i, ], rep(TRUE, K[i]))
             if(itemtype[i] == 'graded')
                 freepars[[i]] <- c(estLambdas[i, ], rep(TRUE, K[i]-1))
@@ -100,63 +102,19 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
             }
         }         
     }
-    for(i in 1:J) names(freepars[[i]]) <- names(startvalues[[i]])
-    if(itemtype[1] == 'Rasch') 
-        for(i in 1:J)
-            startvalues[[i]][1] <- 1/1.702 
+    for(i in 1:J) names(freepars[[i]]) <- names(startvalues[[i]])    
     #load items
     for(i in 1:J){
-        tmp <- c(itemloc[i]:(itemloc[i+1] - 1)) #item location 
-        if(itemtype[i] == 'NullModel' && K[i] == 2){ 
-            pars[[i]] <- new('dich', 
-                             par=startvalues[[i]], 
-                             nfact=1, 
-                             bfactor=BFACTOR,
-                             dat=fulldata[ ,tmp], 
-                             est=freepars[[i]], 
-                             constr=FALSE,
-                             lbound=-25,
-                             ubound=25,
-                             method='Brent',
-                             n.prior.mu=rep(NaN,length(startvalues[[i]])),
-                             n.prior.sd=rep(NaN,length(startvalues[[i]])),
-                             b.prior.alpha=rep(NaN,length(startvalues[[i]])),
-                             b.prior.beta=rep(NaN,length(startvalues[[i]])))
-            tmp2 <- parnumber:(parnumber + length(freepars[[i]]) - 1)            
-            pars[[i]]@parnum <- tmp2
-            parnumber <- parnumber + length(freepars[[i]])            
-        }
-        
-        if(itemtype[i] == 'NullModel' && K[i] > 2){ 
-            pars[[i]] <- new('graded', 
-                             par=startvalues[[i]], 
-                             nfact=1, 
-                             ncat=K[i], 
-                             bfactor=BFACTOR,
-                             dat=fulldata[ ,tmp], 
-                             est=freepars[[i]], 
-                             constr=FALSE,
-                             lbound=-Inf,
-                             ubound=Inf,
-                             method='Nelder-Mead',
-                             n.prior.mu=rep(NaN,length(startvalues[[i]])),
-                             n.prior.sd=rep(NaN,length(startvalues[[i]])),
-                             b.prior.alpha=rep(NaN,length(startvalues[[i]])),
-                             b.prior.beta=rep(NaN,length(startvalues[[i]])))
-            tmp2 <- parnumber:(parnumber + length(freepars[[i]]) - 1)            
-            pars[[i]]@parnum <- tmp2
-            parnumber <- parnumber + length(freepars[[i]])
-        }
+        tmp <- c(itemloc[i]:(itemloc[i+1] - 1)) #item location         
         
         if(any(itemtype[i] == c('Rasch', '1PL')) && K[i] == 2){ 
             pars[[i]] <- new('dich', par=startvalues[[i]], est=freepars[[i]],
                              nfact=nfact, 
                              dat=fulldata[ ,tmp], 
-                             constr=FALSE, 
-                             bfactor=BFACTOR,
+                             constr=TRUE,                              
+                             ncat=2,
                              lbound=ifelse(itemtype[i] == 'Rasch', -25, -Inf),
-                             ubound=ifelse(itemtype[i] == 'Rasch', 25, Inf),
-                             method=ifelse(itemtype[i] == 'Rasch', 'Brent', 'Nelder-Mead'),
+                             ubound=ifelse(itemtype[i] == 'Rasch', 25, Inf),                             
                              n.prior.mu=rep(NaN,length(startvalues[[i]])),
                              n.prior.sd=rep(NaN,length(startvalues[[i]])),
                              b.prior.alpha=rep(NaN,length(startvalues[[i]])),
@@ -167,18 +125,16 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
             parnumber <- parnumber + length(freepars[[i]])            
         }
 
-        if(any(itemtype[i] == c('Rasch', '1PL')) && K[i] > 2){ 
+        if(itemtype[i] == 'Rasch' && K[i] > 2){ 
             pars[[i]] <- new('gpcm', 
                              par=startvalues[[i]], 
                              nfact=nfact, 
                              ncat=K[i],
                              est=freepars[[i]], 
                              dat=fulldata[ ,tmp], 
-                             constr=FALSE, 
-                             bfactor=BFACTOR,
+                             constr=TRUE,                                                           
                              lbound=-Inf,
-                             ubound=Inf,
-                             method='Nelder-Mead',
+                             ubound=Inf,                             
                              n.prior.mu=rep(NaN,length(startvalues[[i]])),
                              n.prior.sd=rep(NaN,length(startvalues[[i]])),
                              b.prior.alpha=rep(NaN,length(startvalues[[i]])),
@@ -190,17 +146,36 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
             parnumber <- parnumber + length(freepars[[i]])            
         }
         
+        if(itemtype[i] == '1PL' && K[i] > 2){
+            pars[[i]] <- new('graded', 
+                             par=startvalues[[i]], 
+                             nfact=nfact, 
+                             ncat=K[i],
+                             est=freepars[[i]], 
+                             dat=fulldata[ ,tmp], 
+                             constr=FALSE,                              
+                             lbound=-Inf,
+                             ubound=Inf,                             
+                             n.prior.mu=rep(NaN,length(startvalues[[i]])),
+                             n.prior.sd=rep(NaN,length(startvalues[[i]])),
+                             b.prior.alpha=rep(NaN,length(startvalues[[i]])),
+                             b.prior.beta=rep(NaN,length(startvalues[[i]])))                   
+            tmp2 <- parnumber:(parnumber + length(freepars[[i]]) - 1)
+            if(length(intersect(tmp2, constr)) > 0 ) pars[[i]]@constr <- TRUE            
+            pars[[i]]@parnum <- tmp2
+            parnumber <- parnumber + length(freepars[[i]])
+        }
+        
         if(any(itemtype[i] == c('2PL', '3PL', '3PLu', '4PL'))){ 
             pars[[i]] <- new('dich', 
                              par=startvalues[[i]], 
                              est=freepars[[i]],
                              nfact=nfact, 
                              dat=fulldata[ ,tmp], 
-                             constr=FALSE, 
-                             bfactor=BFACTOR,
+                             constr=FALSE,                              
+                             ncat=2,
                              lbound=-Inf,                                           
-                             ubound=Inf,
-                             method=ifelse(itemtype[i] == '2PL', 'Nelder-Mead', 'L-BFGS-B'),
+                             ubound=Inf,                             
                              n.prior.mu=rep(NaN,length(startvalues[[i]])),
                              n.prior.sd=rep(NaN,length(startvalues[[i]])),
                              b.prior.alpha=rep(NaN,length(startvalues[[i]])),
@@ -217,6 +192,26 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
             parnumber <- parnumber + length(freepars[[i]])            
         }
         
+        if(any(itemtype[i] == 'grsm')){
+            pars[[i]] <- new('rating', 
+                             par=startvalues[[i]], 
+                             nfact=nfact, 
+                             ncat=K[i],
+                             est=freepars[[i]], 
+                             dat=fulldata[ ,tmp], 
+                             constr=TRUE,                              
+                             lbound=-Inf,
+                             ubound=Inf,                             
+                             n.prior.mu=rep(NaN,length(startvalues[[i]])),
+                             n.prior.sd=rep(NaN,length(startvalues[[i]])),
+                             b.prior.alpha=rep(NaN,length(startvalues[[i]])),
+                             b.prior.beta=rep(NaN,length(startvalues[[i]])))                   
+            tmp2 <- parnumber:(parnumber + length(freepars[[i]]) - 1)
+            if(length(intersect(tmp2, constr)) > 0 ) pars[[i]]@constr <- TRUE            
+            pars[[i]]@parnum <- tmp2
+            parnumber <- parnumber + length(freepars[[i]])
+        }
+        
         if(itemtype[i] == 'graded'){
             pars[[i]] <- new('graded', 
                              par=startvalues[[i]], 
@@ -224,11 +219,9 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              ncat=K[i],
                              est=freepars[[i]], 
                              dat=fulldata[ ,tmp], 
-                             constr=FALSE, 
-                             bfactor=BFACTOR,
+                             constr=FALSE,                              
                              lbound=-Inf,
-                             ubound=Inf,
-                             method='Nelder-Mead',
+                             ubound=Inf,                             
                              n.prior.mu=rep(NaN,length(startvalues[[i]])),
                              n.prior.sd=rep(NaN,length(startvalues[[i]])),
                              b.prior.alpha=rep(NaN,length(startvalues[[i]])),
@@ -246,11 +239,9 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              ncat=K[i],
                              est=freepars[[i]], 
                              dat=fulldata[ ,tmp], 
-                             constr=FALSE, 
-                             bfactor=BFACTOR,
+                             constr=FALSE,                              
                              lbound=-Inf,
-                             ubound=Inf,
-                             method='Nelder-Mead',
+                             ubound=Inf,                             
                              n.prior.mu=rep(NaN,length(startvalues[[i]])),
                              n.prior.sd=rep(NaN,length(startvalues[[i]])),
                              b.prior.alpha=rep(NaN,length(startvalues[[i]])),
@@ -269,11 +260,9 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              nfact=nfact, 
                              ncat=K[i], 
                              dat=fulldata[ ,tmp], 
-                             constr=FALSE, 
-                             bfactor=BFACTOR,
+                             constr=FALSE,                              
                              lbound=-Inf,
-                             ubound=Inf,
-                             method='Nelder-Mead',
+                             ubound=Inf,                             
                              n.prior.mu=rep(NaN,length(startvalues[[i]])),
                              n.prior.sd=rep(NaN,length(startvalues[[i]])),
                              b.prior.alpha=rep(NaN,length(startvalues[[i]])),
@@ -292,11 +281,9 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              est=freepars[[i]],
                              nfact=nfact, 
                              dat=fulldata[ ,tmp], 
-                             constr=FALSE, 
-                             bfactor=BFACTOR,
+                             constr=FALSE,                              
                              lbound=-Inf,
-                             ubound=Inf,
-                             method=ifelse(itemtype[i] == 'PC2PL', 'Nelder-Mead', 'L-BFGS-B'),
+                             ubound=Inf,                             
                              n.prior.mu=rep(NaN,length(startvalues[[i]])),
                              n.prior.sd=rep(NaN,length(startvalues[[i]])),
                              b.prior.alpha=rep(NaN,length(startvalues[[i]])),
@@ -320,11 +307,9 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
                              nfact=nfact, 
                              ncat=K[i], 
                              dat=fulldata[ ,tmp], 
-                             constr=FALSE, 
-                             bfactor=BFACTOR,
+                             constr=FALSE,                              
                              lbound=-Inf,
-                             ubound=Inf,
-                             method='Nelder-Mead',
+                             ubound=Inf,                             
                              n.prior.mu=rep(NaN,length(startvalues[[i]])),
                              n.prior.sd=rep(NaN,length(startvalues[[i]])),
                              b.prior.alpha=rep(NaN,length(startvalues[[i]])),
@@ -355,9 +340,7 @@ LoadPars <- function(itemtype, itemloc, lambdas, zetas, guess, upper, fulldata,
     } 
     attr(pars, 'uniqueconstr') <- constr     
     attr(pars, 'parnumber') <- attr(startvalues, 'parnumber') <- 
-        attr(freepars, 'parnumber') <- parnumber 
-    if(RETURNSTARTVALUES) return(startvalues)
-    if(RETURNFREEPARS) return(freepars)
+        attr(freepars, 'parnumber') <- parnumber     
     return(pars)
 }
 
@@ -374,9 +357,7 @@ LoadGroupPars <- function(gmeans, gcov, estgmeans, estgcov, parnumber, constrain
     est <- c(estgmeans,estgcov[tri])
     names(parnum) <- names(par) <- names(est) <- c(FNMEANS,FNCOV[tri])     
     ret <- new('GroupPars', par=par, est=est, nfact=nfact, 
-               parnum=parnum, method='Nelder-Mead', lbound=-1, ubound=1) #u+lbound diff for MG    
-    if(sum(ret@est) == 1)
-        ret@method <- 'Brent'                  
+               parnum=parnum, lbound=-Inf, ubound=Inf)                      
     if(!is.null(startvalues)){
         if(is.list(startvalues)) ret@par <- startvalues[[length(startvalues)]]
         else return(ret@par)        
