@@ -1,19 +1,18 @@
 #' Item fit statistics
 #' 
-#' \code{itemfit} calculates the Zh values from Drasgow, Levine and Williams (1985) for 
-#' unidimensional and multidimensional models, or \eqn{\chi^2} values for unidimensional models.
+#' \code{itemfit} calculates the Zh values from Drasgow, Levine and Williams (1985), infits, and outfits for 
+#' unidimensional and multidimensional models, and \eqn{\chi^2} values for unidimensional models.
 #' 
 #' 
 #' @aliases itemfit
 #' @param x a computed model object of class \code{ExploratoryClass}, \code{ConfirmatoryClass}, or 
 #' \code{MultipleGroupClass}
-#' @param type a character specifying whether the Zh (\code{'Zh'}) or \eqn{\chi^2} (\code{'X2'}) statistic
-#' should be computed. Not that \code{'X2'} can only be used for unidimensional models
-#' @param ngroups the number of theta groupings to use when computing \code{'X2'}. Cells that have 
-#' any expected values less than 5 are dropped and the degrees of freedom are adjusted accordingly
+#' @param X2 logical; calculate the X2 statistic for unidimensional models?
+#' @param group.size approximate size of each group to be used in calculating the \eqn{\chi^2} statistic
 #' @param empirical.plot a single numeric value indicating which item to plot (via \code{itemplot}) and
 #' overlay with the empirical \eqn{\theta} groupings. Only applicable when \code{type = 'X2'}. 
 #' The default is \code{NULL}, therefore no plots are drawn 
+#' @param degrees the degrees angle to be passed to the \code{\link{iteminfo}} function
 #' @author Phil Chalmers \email{rphilip.chalmers@@gmail.com}
 #' @keywords item fit
 #' @export itemfit
@@ -27,6 +26,9 @@
 #' polychotomous item response models and standardized indices. 
 #' \emph{Journal of Mathematical and Statistical Psychology, 38}, 67-86.
 #' 
+#' Reise, S. P. (1990). A comparison of item- and person-fit methods of assessing model-data fit 
+#' in IRT. \emph{Applied Psychological Measurement, 14}, 127-137.
+#' 
 #' @examples
 #' 
 #' \dontrun{
@@ -38,7 +40,7 @@
 #' items <- rep('dich', 20)
 #' data <- simdata(a,d, 2000, items)
 #'  
-#' x <- mirt(data, 1, SE = FALSE)
+#' x <- mirt(data, 1)
 #' fit <- itemfit(x)
 #' fit
 #' 
@@ -46,23 +48,49 @@
 #' 
 #'   }
 #'
-itemfit <- function(x, type = 'Zh', ngroups = 10, empirical.plot = NULL){
+itemfit <- function(x, X2 = FALSE, group.size = 150, empirical.plot = NULL, degrees = NULL){    
     if(is(x, 'MultipleGroupClass')){
         ret <- list()   
         for(g in 1:length(x@cmods))
-            ret[[g]] <- itemfit(x@cmods[[g]], type=type, ngroups=ngroups)
+            ret[[g]] <- itemfit(x@cmods[[g]], group.size=group.size)
         names(ret) <- names(x@cmods)
         return(ret)
     }    
     sc <- fscores(x, verbose = FALSE, full.scores = TRUE) 
     J <- ncol(x@data)
-    itemloc <- x@itemloc
-    nfact <- x@nfact
+    itemloc <- x@itemloc    
     pars <- x@pars
+    prodlist <- attr(pars, 'prodlist')    
+    nfact <- x@nfact + length(prodlist)
     fulldata <- x@fulldata    
-    Theta <- sc[ ,ncol(sc):(ncol(sc) - nfact + 1), drop = FALSE]
-    if(type == 'X2'){
-        if(nfact > 1) stop('Item chi-squared values are only available for unidimensional models')
+    Theta <- sc[ ,ncol(sc):(ncol(sc) - nfact + 1), drop = FALSE]        
+    N <- nrow(Theta)
+    itemtrace <- matrix(0, ncol=ncol(fulldata), nrow=N)        
+    for (i in 1:J)            
+        itemtrace[ ,itemloc[i]:(itemloc[i+1] - 1)] <- ProbTrace(x=pars[[i]], Theta=Theta)        
+    LL <- itemtrace * fulldata        
+    LL[LL < 1e-8] <- 1
+    Lmatrix <- matrix(log(LL[as.logical(fulldata)]), N, J)              
+    mu <- sigma2 <- rep(0, J)
+    for(item in 1:J){              
+        for(n in 1:N){                
+            P <- itemtrace[n ,itemloc[item]:(itemloc[item+1]-1)]
+            mu[item] <- mu[item] + sum(P * log(P))            
+            for(i in 1:length(P)){
+                for(j in 1:length(P)){
+                    if(i != j)
+                        sigma2[item] <- sigma2[item] + P[i] * P[j] * log(P[i]) * log(P[i]/P[j])
+                }
+            }                               
+        }               
+    }        
+    Zh <- (colSums(Lmatrix) - mu) / sqrt(sigma2)    
+    attr(x, 'inoutfitreturn') <- TRUE
+    pf <- personfit(x, degrees=degrees)
+    outfit <- colSums(pf$Z^2) / J
+    infit <- colSums(pf$Z^2 * pf$info) / colSums(pf$info)        
+    ret <- data.frame(item=colnames(x@data), outfit=outfit, infit=infit, Zh=Zh)
+    if((X2 || !is.null(empirical.plot)) && nfact == 1){                
         ord <- order(Theta[,1])    
         fulldata <- fulldata[ord,]
         Theta <- Theta[ord, , drop = FALSE]
@@ -70,6 +98,7 @@ itemfit <- function(x, type = 'Zh', ngroups = 10, empirical.plot = NULL){
         den <- den / sum(den)
         cumTheta <- cumsum(den)
         Groups <- rep(20, length(ord))
+        ngroups <- ceiling(nrow(fulldata) / group.size)        
         weight <- 1/ngroups        
         for(i in 1:20)
             Groups[round(cumTheta,2) >= weight*(i-1) & round(cumTheta,2) < weight*i] <- i        
@@ -83,48 +112,24 @@ itemfit <- function(x, type = 'Zh', ngroups = 10, empirical.plot = NULL){
                 r <- colSums(dat)
                 N <- nrow(dat)                  
                 mtheta <- matrix(mean(Theta[Groups == j,]), nrow=1)
-                if(!is.null(empirical.plot))
-                    points(rep(mtheta, length(r)), r/N)               
+                if(!is.null(empirical.plot)){
+                    tmp <- r/N
+                    col <- 2:length(tmp)
+                    points(rep(mtheta, length(tmp) - 1), tmp[-1], col = col)                  
+                }
                 P <- ProbTrace(x=pars[[i]], Theta=mtheta)            
                 if(any(N * P < 2)){
                     df[i] <- df[i] - 1
                     next
-                }                
+                }                                
                 X2[i] <- X2[i] + sum((r - N*P)^2 / N*P)
-            }
+            }            
             df[i] <- df[i] + n.uniqueGroups*(length(r) - 1) - sum(pars[[i]]@est)            
         }
+        X2[X2 == 0] <- NA
         if(!is.null(empirical.plot)) return(invisible(NULL))
-        p <- pchisq(X2, df, lower.tail = FALSE)               
-        ret <- data.frame(item=colnames(x@data), X2=X2, df=df, p=round(p,3)) 
-        return(ret)
-    }    
-    if(type == 'Zh'){
-        N <- nrow(Theta)
-        itemtrace <- matrix(0, ncol=ncol(fulldata), nrow=N)        
-        for (i in 1:J){
-            if(x@pars[[1]]@bfactor) pars[[i]]@bfactor <- FALSE
-            itemtrace[ ,itemloc[i]:(itemloc[i+1] - 1)] <- ProbTrace(x=pars[[i]], Theta=Theta)
-        }
-        LL <- itemtrace * fulldata        
-        LL[LL < 1e-8] <- 1
-        Lmatrix <- matrix(log(LL[as.logical(fulldata)]), N, J)              
-        mu <- sigma2 <- rep(0, J)
-        for(item in 1:J){              
-            for(n in 1:N){                
-                P <- itemtrace[n ,itemloc[item]:(itemloc[item+1]-1)]
-                mu[item] <- mu[item] + sum(P * log(P))            
-                for(i in 1:length(P)){
-                    for(j in 1:length(P)){
-                        if(i != j)
-                            sigma2[item] <- sigma2[item] + P[i] * P[j] * log(P[i]) * log(P[i]/P[j])
-                    }
-                }                               
-            }               
-        }        
-        Zh <- (colSums(Lmatrix) - mu) / sqrt(sigma2)
-        p <- round(pnorm(Zh), 3)
-        ret <- data.frame(item=colnames(x@data), Zh=Zh, p=p)                
-        return(ret)
-    }    
+        ret$df <- df
+        ret$X2 <- X2
+    }                    
+    return(ret)
 }
