@@ -71,28 +71,26 @@
 #' of exploratory factors to estimate. See \code{\link{confmirt.model}} for
 #' more details
 #' @param itemtype type of items to be modeled, declared as a vector for each item or a single value
-#' which will be repeated globally. The NULL default assumes that the items are ordinal or 2PL,
+#' which will be repeated globally. The NULL default assumes that the items follow a graded or 2PL structure,
 #' however they may be changed to the following: 'Rasch', '1PL', '2PL', '3PL', '3PLu', 
-#' '4PL', 'graded', 'gpcm', 'nominal', 'mcm', and 'partcomp', for the Rasch/partial credit, 1 and 2 parameter logistic, 
+#' '4PL', 'graded', 'grsm', 'gpcm', 'nominal', 'mcm', 'PC2PL', and 'PC3PL', for the Rasch/partial credit, 1 and 2 parameter logistic, 
 #' 3 parameter logistic (lower asymptote and upper), 4 parameter logistic, graded response model, 
-#' generalized partial credit model, nominal model, multiple choice model, and partially compensatory model,
-#' respectively. If \code{NULL} the default assumes that the data follow a '2PL' or 'graded' format
+#' rating scale graded response model, generalized partial credit model, nominal model, 
+#' multiple choice model, and 2-3PL partially compensatory model, respectively 
+#' @param grsm.block an optional numeric vector indicating where the blocking should occur when using 
+#' the grsm, NA represents items that do not belong to the grsm block (other items that may be estimated
+#' in the test data). For example, to specify two blocks of 3 with a 2PL item for the last item:
+#' \code{grsm.block = c(rep(1,3), rep(2,3), NA)}. If NULL the all items are assumed to be within the same 
+#' group and therefore have the same number of item categories
 #' @param SE logical, estimate the standard errors? Calls the MHRM subroutine for a stochastic approximation
+#' @param SEtol tollerance value used to stop the MHRM estimation when \code{SE = TRUE}. Lower values
+#' will take longer but may be more stable for computing the information matrix
 #' @param guess fixed pseudo-guessing parameters. Can be entered as a single
 #' value to assign a global guessing parameter or may be entered as a numeric
 #' vector corresponding to each item
 #' @param upper fixed upper bound parameters for 4-PL model. Can be entered as a single
 #' value to assign a global guessing parameter or may be entered as a numeric
 #' vector corresponding to each item
-#' @param free.start a list containing the start value and logical indicating whether a given parameter 
-#' is to be freely estimated. Each element of the list consists of three components, the parameter
-#' number, the starting (or fixed) value, and a logical to indicate whether the parameter is free. For
-#' example, \code{free.start = list(c(20,0,FALSE), c(10,1.5,TRUE))} would fix parameter 20 to 0 while
-#' parameter 10 would be freely estimated with a starting value of 1.5. Note that this will override 
-#' the values specified by a user defined \code{startvalues} or \code{freepars} input for the specified
-#' parameters
-#' 
-#' 
 #' @param prev.cor use a previously computed correlation matrix to be used to
 #' estimate starting values for the EM estimation? Default in \code{NULL} 
 #' @param rotate type of rotation to perform after the initial orthogonal
@@ -103,27 +101,21 @@
 #' @param allpars logical; print all the item parameters instead of just the slopes?
 #' @param Target a dummy variable matrix indicting a target rotation pattern
 #' @param constrain a list of user declared equality constraints. To see how to define the
-#' parameters correctly use \code{constrain = 'index'} initially to see how the parameters are labeled.
+#' parameters correctly use \code{pars = 'values'} initially to see how the parameters are labeled.
 #' To constrain parameters to be equal create a list with separate concatenated vectors signifying which
 #' parameters to constrain. For example, to set parameters 1 and 5 equal, and also set parameters 2, 6, and 10 equal
 #' use \code{constrain = list(c(1,5), c(2,6,10))}
 #' @param parprior a list of user declared prior item probabilities. To see how to define the
-#' parameters correctly use \code{parprior = 'index'} initially to see how the parameters are labeled.
+#' parameters correctly use \code{pars = 'values'} initially to see how the parameters are labeled.
 #' Can define either normal (normally for slopes and intercepts) or beta (for guessing and upper bounds) prior
 #' probabilities. Note that for upper bounds the value used in the prior is 1 - u so that the lower and upper 
 #' bounds can function the same. To specify a prior the form is c('priortype', ...), where normal priors 
 #' are \code{parprior = list(c(parnumber, 'norm', mean, sd))} and betas are 
 #' \code{parprior = list(c(parnumber, 'beta', alpha, beta))}. 
-#' @param freepars a list of user declared logical values indicating which parameters to estimate. 
-#' To see how to define the parameters correctly use \code{freepars = 'index'} initially to see how the parameters
-#' are labeled. These values may be modified and input back into the function by using 
-#' \code{freepars=newfreepars}. Note that user input values must match what the default structure 
-#' would have been
-#' @param startvalues a list of user declared start values for parameters. To see how to define the
-#' parameters correctly use \code{startvalues = 'index'} initially to see what the defaults would 
-#' noramlly be. These values may be modified and input back into the function by using 
-#' \code{startavlues=newstartvalues}. Note that user input values must match what the default structure 
-#' would have been
+#' @param pars a data.frame with the structure of how the starting values, parameter numbers, and estimation
+#' logical values are defined. The user may observe how the model defines the values by using \code{pars = 
+#' 'values'}, and this object can in turn be modified and input back into the estimation with \code{pars = 
+#' mymodifiedpars}
 #' @param quadpts number of quadrature points per dimension
 #' @param printvalue a numeric value to be specified when using the \code{res='exp'}
 #' option. Only prints patterns that have standardized residuals greater than 
@@ -150,6 +142,7 @@
 #' @param restype type of residuals to be displayed. Can be either \code{'LD'}
 #' for a local dependence matrix (Chen & Thissen, 1997) or \code{'exp'} for the
 #' expected values for the frequencies of every response pattern
+#' @param df.p logical; print the degrees of freedom and p-values?
 #' @param verbose logical; print observed log-likelihood value at each iteration?
 #' @param debug logical; turn on debugging features?
 #' @param technical a list containing lower level technical parameters for estimation. May be:
@@ -178,8 +171,8 @@
 #' 
 #' \describe{ 
 #' \item{Rasch}{
-#' Only one intercept estimated. Note the 1.702 constant. \deqn{P(x = 1|\theta, d) = \frac{1}{1 + 
-#' exp(-1.702 * (\theta + d}))}
+#' Only one intercept estimated. \deqn{P(x = 1|\theta, d) = \frac{1}{1 + 
+#' exp(-1*(\theta + d}))}
 #' }
 #' \item{1-4PL}{
 #' Depending on the model \eqn{u} may be equal to 1 and \eqn{g} may be equal to 0. 
@@ -191,6 +184,13 @@
 #' the predicted category. 
 #' \deqn{P(x = k | \theta, \psi) = P(x \ge k | \theta, \phi) - P(x \ge k + 1 | \theta, \phi)}
 #' }
+#' \item{grsm}{
+#' A more constrained version of the graded model where graded spacing is equal accross item blocks
+#' and only adjusted by a single 'difficulty' parameter (c). Again,
+#' \deqn{P(x = k | \theta, \psi) = P(x \ge k | \theta, \phi) - P(x \ge k + 1 | \theta, \phi)}#' 
+#' but now 
+#' \deqn{P = \frac{1}{1 + exp(-1.702 * (a_1 * \theta_1 + a_2 * \theta_2 + d_k + c}))} 
+#' } 
 #' \item{gpcm/nominal}{For the gpcm the \eqn{d_k} values are treated as fixed and orderd values 
 #' from 0:(k-1) (in the nominal model \eqn{d_0} is also set to 0). Additionally, for identification 
 #' in the nominal model \eqn{ak_0 = 1}, \eqn{ak_k = (k - 1)}.
@@ -246,16 +246,15 @@
 #' IL: Scientific Software International.
 #' @keywords models
 #' @usage 
-#' mirt(data, model, itemtype = NULL, guess = 0, upper = 1, SE = TRUE, free.start = NULL, 
-#' startvalues = NULL,
-#' constrain = NULL, freepars = NULL,  parprior = NULL, rotate = 'varimax', Target = NULL, 
-#' prev.cor = NULL, quadpts = NULL, verbose = FALSE, debug = FALSE, 
+#' mirt(data, model, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SEtol = .001, pars = NULL, 
+#' constrain = NULL, parprior = NULL, rotate = 'varimax', Target = NaN, 
+#' prev.cor = NULL, quadpts = NULL, grsm.block = NULL, verbose = FALSE, debug = FALSE, 
 #' technical = list(), ...)
 #' 
 #' \S4method{summary}{ExploratoryClass}(object, rotate = '', Target = NULL, suppress = 0, digits = 3, 
 #' verbose = TRUE, ...)
 #' 
-#' \S4method{coef}{ExploratoryClass}(object, rotate = '', Target = NULL, allpars = FALSE, digits = 3, 
+#' \S4method{coef}{ExploratoryClass}(object, rotate = '', Target = NULL, allpars = TRUE, digits = 3, 
 #' verbose = TRUE, ...)
 #' 
 #' \S4method{anova}{ExploratoryClass}(object, object2)
@@ -265,7 +264,7 @@
 #' \S4method{plot}{ExploratoryClass}(x, y, type = 'info', npts = 50, theta_angle = 45, 
 #' rot = list(xaxis = -70, yaxis = 30, zaxis = 10), ...)
 #' 
-#' \S4method{residuals}{ExploratoryClass}(object, restype = 'LD', digits = 3, printvalue = NULL, 
+#' \S4method{residuals}{ExploratoryClass}(object, restype = 'LD', digits = 3, df.p = FALSE, printvalue = NULL, 
 #' verbose = TRUE, ...)
 #' @export mirt
 #' @examples
@@ -282,9 +281,9 @@
 #' 
 #' #estimated 3PL model for item 5 only
 #' (mod1.3PL <- mirt(data, 1, itemtype = c('2PL', '2PL', '2PL', '2PL', '3PL')))
-#' coef(mod1.3PL, allpars = TRUE)
+#' coef(mod1.3PL)
 #' 
-#' (mod2 <- mirt(data, 2))
+#' (mod2 <- mirt(data, 2, SE = TRUE))
 #' summary(mod2, rotate = 'oblimin')
 #' coef(mod2)
 #' residuals(mod2)
@@ -305,20 +304,19 @@
 #' 
 #' ###########
 #' #data from the 'ltm' package in numeric format
-#' pmod1 <- mirt(Science, 1, SE = FALSE)
+#' pmod1 <- mirt(Science, 1)
 #' plot(pmod1)
 #' summary(pmod1)
 #'
 #' #Constrain all slopes to be equal
 #' #first obtain parameter index
-#' mirt(Science,1, constrain = 'index') #note that slopes are numbered 1,5,9,13
+#' values <- mirt(Science,1, pars = 'values') 
+#' values #note that slopes are numbered 1,5,9,13
 #' (pmod1_equalslopes <- mirt(Science, 1, constrain = list(c(1,5,9,13))))
 #' coef(pmod1_equalslopes)
-#' #manually fix the first slope to .6
-#' (pmod1_fixedslope <- mirt(Science, 1, free.start = list(c(1, .6, FALSE))))
 #' 
 #' pmod2 <- mirt(Science, 2)
-#' coef(pmod2)
+#' summary(pmod2)
 #' residuals(pmod2)
 #' plot(pmod2, theta_angle = seq(0,90, by = 5)) #sum across angles of theta 1
 #' itemplot(pmod2, 1)
@@ -343,80 +341,39 @@
 #' coef(mod2g)
 #' anova(mod1g, mod2g)
 #' summary(mod2g, rotate='promax')
+#' 
+#' ###########
+#' #graded rating scale example
+#' 
+#' #make some data
+#' a <- matrix(rep(1/1.702, 10))
+#' d <- matrix(c(1,0.5,-.5,-1), 10, 4, byrow = TRUE)
+#' c <- seq(-1, 1, length.out=10)
+#' data <- simdata(a, d + c, 2000, itemtype = rep('graded',10))
+#'
+#' #use much better start values to save iterations
+#' sv <- mirt(data, 1, itemtype = 'grsm', pars = 'values')
+#' sv[,5] <- c(as.vector(t(cbind(a,d,c))),0,1) 
+#'
+#' mod1 <- mirt(data, 1)
+#' mod2 <- mirt(data, 1, itemtype = 'grsm', verbose = TRUE, pars = sv)
+#' coef(mod2)
+#' anova(mod2, mod1) #not sig, mod2 should be prefered 
 #' }
 #' 
-mirt <- function(data, model, itemtype = NULL, guess = 0, upper = 1, SE = TRUE, free.start = NULL, 
-                 startvalues = NULL,
-                 constrain = NULL, freepars = NULL,  parprior = NULL, rotate = 'varimax', Target = NULL, 
-                 prev.cor = NULL, quadpts = NULL, verbose = FALSE, debug = FALSE, 
-                 technical = list(), ...)
+mirt <- function(data, model, itemtype = NULL, guess = 0, upper = 1, SE = FALSE, SEtol = .001,
+                  pars = NULL, constrain = NULL, parprior = NULL, rotate = 'varimax', Target = NaN, 
+                  prev.cor = NULL, quadpts = NULL, grsm.block = NULL, verbose = FALSE, debug = FALSE, 
+                  technical = list(), ...)
 {   
     if(debug == 'Main') browser()
-	Call <- match.call()    
-    ##technical
-    MAXQUAD <- ifelse(is.null(technical$MAXQUAD), 10000, technical$MAXQUAD)
-    MSTEPMAXIT <- ifelse(is.null(technical$MSTEPMAXIT), 25, technical$MSTEPMAXIT)
-	TOL <- ifelse(is.null(technical$TOL), .001, technical$TOL)
-	NCYCLES <- ifelse(is.null(technical$NCYCLES), 300, technical$NCYCLES)
-    NOWARN <- ifelse(is.null(technical$NOWARN), TRUE, technical$NOWARN)        	       
-    RETURN <- ifelse(any('index' == c(startvalues, freepars, parprior, constrain)), TRUE, FALSE)
-    ##              
-    Target <- ifelse(is.null(Target), NaN, Target)   
-    data <- as.matrix(data)
-    PrepList <- PrepData(data=data, model=model, itemtype=itemtype, guess=guess, upper=upper, 
-                         startvalues=startvalues, constrain=constrain, freepars=freepars, 
-                         parprior=parprior, verbose=verbose, free.start=free.start, debug=debug, 
-                         technical=technical)
-    if(RETURN) return(PrepList)
-    NULL.MODEL <- ifelse(PrepList$itemtype[1] == 'NullModel', TRUE, FALSE)    
-    nfact <- PrepList$nfact
-    nLambdas <- PrepList$nLambdas
-    if(nLambdas > nfact) stop('Polynominals and product terms not supported for EM method')
-	if (is.null(quadpts)) quadpts <- ceiling(40/(PrepList$nfact^1.5))  
-	Theta <- theta <- as.matrix(seq(-4,4,length.out = quadpts))
-	if(quadpts^nfact <= MAXQUAD){
-		Theta <- thetaComb(theta,nfact)		
-	} else stop('Greater than ', MAXQUAD, ' quadrature points.')        
-    ESTIMATE <- EM(pars=PrepList$pars, NCYCLES=NCYCLES, MSTEPMAXIT=MSTEPMAXIT, TOL=TOL,                    
-                   tabdata=PrepList$tabdata, tabdata2=PrepList$tabdata2, npars=PrepList$npars,
-                   Theta=Theta, theta=theta, itemloc=PrepList$itemloc, debug=debug, verbose=verbose, 
-                   constrain=PrepList$constrain, data=data, NULL.MODEL=NULL.MODEL)	
-	# pars to FA loadings
-    pars <- ESTIMATE$pars    
-    lambdas <- Lambdas(pars)
-	if (nLambdas > 1) norm <- sqrt(1 + rowSums(lambdas[ ,1:nLambdas]^2))
-		else norm <- as.matrix(sqrt(1 + lambdas[ ,1]^2))  
-	alp <- as.matrix(lambdas[ ,1:nLambdas]/norm)
-    if(PrepList$exploratory){
-    	FF <- alp %*% t(alp)
-    	V <- eigen(FF)$vector[ ,1:nfact]
-    	L <- eigen(FF)$values[1:nfact]
-    	if (nfact == 1) F <- as.matrix(V * sqrt(L))
-    		else F <- V %*% sqrt(diag(L))  
-    	if (sum(F[ ,1] < 0)) F <- (-1) * F 
-    	colnames(F) <- paste("F_", 1:ncol(F),sep="")
-    	h2 <- rowSums(F^2)
-    	mod <- new('ExploratoryClass', iter=ESTIMATE$cycles, pars=ESTIMATE$pars, G2=ESTIMATE$G2, 
-    	           df=ESTIMATE$df, p=ESTIMATE$p, itemloc=PrepList$itemloc, AIC=ESTIMATE$AIC, 
-    	           BIC=ESTIMATE$BIC, logLik=ESTIMATE$logLik, F=F, h2=h2, tabdata=PrepList$tabdata2, 
-    	           Theta=Theta, Pl=ESTIMATE$Pl, data=data, converge=ESTIMATE$converge, nfact=nfact,               
-    	           quadpts=quadpts, RMSEA=ESTIMATE$RMSEA, K=PrepList$K, tabdatalong=PrepList$tabdata, 
-    	           rotate=rotate, null.mod=ESTIMATE$null.mod, TLI=ESTIMATE$TLI, Target=Target, 
-    	           factorNames=PrepList$factorNames, constrain=PrepList$constrain, 
-    	           fulldata=PrepList$fulldata, Call=Call)
-    } else {
-        F <- alp
-        colnames(F) <- PrepList$factorNames    
-        h2 <- rowSums(F^2)       
-        mod <- new('ConfirmatoryClass', iter=ESTIMATE$cycles, pars=ESTIMATE$pars, G2=ESTIMATE$G2, 
-                   df=ESTIMATE$df, p=ESTIMATE$p, itemloc=PrepList$itemloc, AIC=ESTIMATE$AIC, 
-                   BIC=ESTIMATE$BIC, logLik=ESTIMATE$logLik, F=F, h2=h2, tabdata=PrepList$tabdata2, 
-                   Theta=Theta, Pl=ESTIMATE$Pl, data=data, converge=ESTIMATE$converge, nfact=nfact,               
-                   quadpts=quadpts, RMSEA=ESTIMATE$RMSEA, K=PrepList$K, tabdatalong=PrepList$tabdata, 
-                   null.mod=ESTIMATE$null.mod, TLI=ESTIMATE$TLI, factorNames=PrepList$factorNames, 
-                   constrain=PrepList$constrain, fulldata=PrepList$fulldata, Call=Call)
-    }   
-    if(SE) mod <- calcEMSE(object=mod, data=data, model=model, constrain=constrain, 
-                           parprior=parprior, verbose=verbose)
-	return(mod)    
+    Call <- match.call()    
+    mod <- ESTIMATION(data=data, model=model, group=rep('all', nrow(data)), 
+                      itemtype=itemtype, guess=guess, upper=upper, grsm.block=grsm.block,
+                      pars=pars, method = 'EM', constrain=constrain, SE=SE, SEtol=SEtol,
+                      parprior=parprior, quadpts=quadpts, rotate=rotate, Target=Target,
+                      technical = technical, debug = debug, verbose = verbose, ...)
+    if(is(mod, 'ExploratoryClass') || is(mod, 'ConfirmatoryClass'))
+        mod@Call <- Call
+    return(mod)    
 }
