@@ -7,8 +7,8 @@ thetaComb <- function(theta, nfact)
 	else if (nfact == 4) Theta <- expand.grid(theta,theta,theta,theta)
 	else if (nfact == 5) Theta <- expand.grid(theta,theta,theta,theta,theta)        	
 	else if (nfact == 6) Theta <- expand.grid(theta,theta,theta,theta,theta,theta)
-	if(nfact > 6) stop('Are you crazy?!?!? That\'s way too many factors for this quandrature method.
-                       Try using confmirt() instead for better accuracy')
+	if(nfact > 6) stop('Are you crazy?!?!? That is way too many factors for this quadrature method.
+                       Try using confmirt() instead for better numerical accuracy')
 	Theta <- as.matrix(Theta)	
 	return(Theta)     
 }
@@ -280,30 +280,31 @@ reloadConstr <- function(par, constr, obj){
     return(obj)
 }
 
-calcEMSE <- function(object, data, model, constrain, parprior, verbose){  
-    startvalues <- list()
-    for(i in 1:(ncol(data)+1))
-        startvalues[[i]] <- object@pars[[i]]@par
-    if(is(model, 'numeric') && length(model) > 1){
-        J <- ncol(data)
-        tmp <- tempfile('tempfile')
-        unique <- unique(model)
-        index <- 1:J
-        tmp2 <- sprintf(c('G =', paste('1-', J, sep='')))
-        for(i in 1:length(unique)){
-            ind <- index[model == unique[i]]
-            comma <- rep(',', 2*length(ind))
-            TF <- rep(c(TRUE,FALSE), length(ind))
-            comma[TF] <- ind
-            comma[length(comma)] <- ""
-            tmp2 <- c(tmp2, c(paste('\nF', i, ' =', sep=''), comma))
-        }
-        cat(tmp2, file=tmp)
-        model <- confmirt.model(tmp, quiet = TRUE)        
-        unlink(tmp)
+bfactor2mod <- function(model, J){        
+    tmp <- tempfile('tempfile')
+    unique <- sort(unique(model))
+    index <- 1:J
+    tmp2 <- sprintf(c('G =', paste('1-', J, sep='')))
+    for(i in 1:length(unique)){
+        ind <- index[model == unique[i]]
+        comma <- rep(',', 2*length(ind))
+        TF <- rep(c(TRUE,FALSE), length(ind))
+        comma[TF] <- ind
+        comma[length(comma)] <- ""
+        tmp2 <- c(tmp2, c(paste('\nF', i, ' =', sep=''), comma))
     }
-    pars <- confmirt(data, model, startvalues=startvalues, constrain=constrain, parprior=parprior, 
-                    technical = list(BURNIN = 1, SEMCYCLES = 5, TOL = .01, 
+    cat(tmp2, file=tmp)
+    model <- confmirt.model(tmp, quiet = TRUE)        
+    unlink(tmp)
+    return(model)
+}
+
+calcEMSE <- function(object, data, model, itemtype, fitvalues, constrain, parprior, verbose){          
+    if(is(model, 'numeric') && length(model) > 1)
+        model <- bfactor2mod(model, data)
+    pars <- confmirt(data, model, itemtype=itemtype, pars=fitvalues, constrain=constrain, 
+                     parprior=parprior, 
+                     technical = list(BURNIN = 1, SEMCYCLES = 5, TOL = .01, 
                                     EMSE = TRUE), verbose = verbose)    
     for(i in 1:length(pars$pars))
         object@pars[[i]]@SEpar <- pars$pars[[i]]@SEpar                            
@@ -312,7 +313,11 @@ calcEMSE <- function(object, data, model, constrain, parprior, verbose){
     return(object)
 }
 
-UpdateConstrain <- function(pars, constrain, invariance, nfact, nLambdas, J, ngroups){
+UpdateConstrain <- function(pars, constrain, invariance, nfact, nLambdas, J, ngroups, PrepList){    
+    for(g in 1:ngroups)  
+        if(length(PrepList[[g]]$constrain) > 0)
+            for(i in 1:length(PrepList[[g]]$constrain))
+                constrain[[length(constrain) + 1]] <- PrepList[[g]]$constrain[[i]]        
     if('covariances' %in% invariance){ #Fix covariance accross groups (only makes sense with vars = 1)
         tmpmat <- matrix(NA, nfact, nfact)
         low_tri <- lower.tri(tmpmat)
@@ -375,3 +380,39 @@ UpdateConstrain <- function(pars, constrain, invariance, nfact, nLambdas, J, ngr
     }
     return(constrain)
 }
+
+ReturnPars <- function(PrepList, itemnames, MG = FALSE){    
+    parnum <- par <- est <- item <- parname <- gnames <- itemtype <- c()                                    
+    if(!MG) PrepList <- list(full=PrepList)                        
+    for(g in 1:length(PrepList)){
+        tmpgroup <- PrepList[[g]]$pars                                
+        for(i in 1:length(tmpgroup)){
+            if(i <= length(itemnames))
+                item <- c(item, rep(itemnames[i], length(tmpgroup[[i]]@parnum)))
+            parname <- c(parname, names(tmpgroup[[i]]@parnum))
+            parnum <- c(parnum, tmpgroup[[i]]@parnum) 
+            par <- c(par, tmpgroup[[i]]@par)
+            est <- c(est, tmpgroup[[i]]@est)                    
+        }
+        item <- c(item, rep('GROUP', length(tmpgroup[[i]]@parnum)))                                
+    }
+    gnames <- rep(names(PrepList), each = length(est)/length(PrepList))
+    ret <- data.frame(group=gnames, item = item, name=parname, parnum=parnum, value=par, est=est)
+    ret
+}
+
+UpdatePrepList <- function(PrepList, pars, MG = FALSE){
+    if(!MG) PrepList <- list(PrepList)
+    ind <- 1    
+    for(g in 1:length(PrepList)){
+        for(i in 1:length(PrepList[[g]]$pars)){ 
+            for(j in 1:length(PrepList[[g]]$pars[[i]]@par)){
+                PrepList[[g]]$pars[[i]]@par[j] <- pars[ind,5]
+                PrepList[[g]]$pars[[i]]@est[j] <- as.logical(pars[ind,6])                
+                ind <- ind + 1
+            }
+        }
+    }    
+    if(!MG) PrepList <- PrepList[[1]]
+    return(PrepList)
+}
